@@ -2,7 +2,7 @@
  *  
  *  Copyright (C) 2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
  *  Copyright (C) 2003,2006 Benedikt Meurer <benny@xfce.org>
- *  Copyright (c) 2004 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2006 Brian Tarricone <bjt23@cornell.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -57,6 +57,10 @@
 #endif
 #include <gtk/gtk.h>
 
+#ifdef HAVE_LIBEXO
+#include <exo/exo.h>
+#endif
+
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <xfce-mcs-manager/manager-plugin.h>
@@ -275,9 +279,9 @@ backdrop_create_channel (McsPlugin * mcs_plugin)
             if(setting)
                 bp->show_image = setting->data.v_int == 0 ? FALSE : TRUE;
             else {
-                bp->show_image = TRUE;
+                bp->show_image = FALSE;
                 mcs_manager_set_int(mcs_plugin->manager, setting_name,
-                        BACKDROP_CHANNEL, 1);
+                        BACKDROP_CHANNEL, 0);
             }
             
             /* the color style */
@@ -287,7 +291,7 @@ backdrop_create_channel (McsPlugin * mcs_plugin)
             if(setting)
                 bp->color_style = setting->data.v_int;
             else {
-                bp->color_style = XFCE_BACKDROP_COLOR_HORIZ_GRADIENT;
+                bp->color_style = XFCE_BACKDROP_COLOR_SOLID;
                 mcs_manager_set_int(mcs_plugin->manager, setting_name,
                         BACKDROP_CHANNEL, bp->color_style);
             }
@@ -394,50 +398,6 @@ showimage_toggle(GtkWidget *b, BackdropPanel *bp)
 }
 
 /* dnd box */
-void
-on_drag_data_received (GtkWidget * w, GdkDragContext * context,
-               int x, int y, GtkSelectionData * data,
-               guint info, guint time, BackdropPanel *bp)
-{
-    char buf[1024];
-    char *file = NULL;
-    char *end;
-
-    /* copy data to buffer */
-    strncpy (buf, (char *) data->data, 1023);
-    buf[1023] = '\0';
-
-    if ((end = strchr (buf, '\n')))
-    *end = '\0';
-
-    if ((end = strchr (buf, '\r')))
-    *end = '\0';
-
-    if (buf[0])
-    {
-    file = buf;
-
-    if (strncmp ("file:", file, 5) == 0)
-    {
-        file += 5;
-
-        if (strncmp ("///", file, 3) == 0)
-        file += 2;
-    }
-
-    if(bp->image_path)
-        g_free (bp->image_path);
-    bp->image_path = g_strdup (file);
-
-    gtk_entry_set_text (GTK_ENTRY (bp->file_entry), bp->image_path);
-    gtk_editable_set_position (GTK_EDITABLE (bp->file_entry), -1);
-
-    update_path (bp);
-    }
-
-    gtk_drag_finish (context, (file != NULL),
-             (context->action == GDK_ACTION_MOVE), time);
-}
 
 /* Don't use 'text/plain' as target.
  * Otherwise backdrop lists can not be dropped
@@ -453,6 +413,88 @@ static GtkTargetEntry target_table[] = {
     {"text/uri-list", 0, TARGET_URL},
 };
 
+#define VALID_HEXDIGIT(d) ( ((d) >= '0' &&  (d) <= '9') \
+                            || ((d) >= 'a' && (d) <= 'f') \
+                            || ((d) >= 'A' && (d) <= 'F') )
+
+void
+on_drag_data_received (GtkWidget * w, GdkDragContext * context,
+               int x, int y, GtkSelectionData * data,
+               guint info, guint time, BackdropPanel *bp)
+{
+    gchar *file = NULL, *p, *q, hexdigit[3] = { 0, 0, 0 };
+    
+    if(TARGET_STRING == info) {
+        /* it should NOT be a uri: just assume the string is a filename */
+        file = g_strndup((gchar *)data->data, data->length);
+        while(file[strlen(file)-1] == '\n' || file[strlen(file)-1] == '\r')
+            file[strlen(file)-1] = 0;
+    } else if(TARGET_URL == info) {
+        if(data->length > PATH_MAX - 1) {
+            g_critical("File name longer than %d chars dropped.", PATH_MAX);
+            gtk_drag_finish(context, FALSE, FALSE, time);
+            return;
+        }
+        
+        /* this assumes that the converted version of data->data will be at most
+         * equal to the original length of data. */
+        file = g_malloc0(data->length+1);
+        
+        p = (gchar *)data->data;
+        if(!strncmp(p, "file:", 5)) {
+            p += 5;
+            if(!strncmp(p, "///", 3))
+                p += 2;
+        }
+        
+        for(q = file; p && *p; ++q) {
+            switch(*p) {
+                case '%':
+                    if(!VALID_HEXDIGIT(*(p+1)) || !VALID_HEXDIGIT(*(p+2))) {
+                        g_critical("Dropped text/uri-list filename is an invalid URI.");
+                        gtk_drag_finish(context, FALSE, FALSE, time);
+                        return;
+                    }
+                    
+                    hexdigit[0] = *(p+1);
+                    hexdigit[1] = *(p+2);
+                    *q = strtol(hexdigit, NULL, 16);
+                    
+                    p += 3;
+                    break;
+                
+                case '\r':
+                case '\n':
+                    /* this uri list might have more than one element.  for now,
+                     * just take the first element and be done with it. */
+                    p = NULL;
+                    break;
+                
+                default:
+                    *q = *p;
+                    ++p;
+                    break;
+            }
+        }
+        
+        /* FIXME: does |file| need to be passed through
+         * g_filename_from_utf8()? */
+    }
+    
+    if(file) {
+        g_free(bp->image_path);
+        bp->image_path = file;
+        
+        gtk_entry_set_text(GTK_ENTRY(bp->file_entry), bp->image_path);
+        gtk_editable_set_position(GTK_EDITABLE(bp->file_entry), -1);
+
+        update_path(bp);
+    }
+    
+    gtk_drag_finish(context, (file != NULL),
+                    (context->action == GDK_ACTION_MOVE), time);
+}
+
 static void
 set_dnd_dest (BackdropPanel * bp)
 {
@@ -496,6 +538,7 @@ file_entry_lost_focus (GtkWidget * entry, GdkEventFocus * ev,
     return FALSE;
 }
 
+#ifndef HAVE_LIBEXO
 static void
 update_preview_cb(GtkFileChooser *chooser, gpointer data)
 {
@@ -516,11 +559,15 @@ update_preview_cb(GtkFileChooser *chooser, gpointer data)
     }
     gtk_file_chooser_set_preview_widget_active(chooser, (pix != NULL));
 }
+#endif
 
 static void
 browse_cb(GtkWidget *b, BackdropPanel *bp)
 {
-    GtkWidget *chooser, *preview;
+    GtkWidget *chooser;
+#ifndef HAVE_LIBEXO
+    GtkWidget *preview;
+#endif
     GtkFileFilter *filter;
     gchar *confdir;
     
@@ -565,12 +612,16 @@ browse_cb(GtkWidget *b, BackdropPanel *bp)
         g_free(tmppath);
     }
     
+#ifdef HAVE_LIBEXO
+    exo_gtk_file_chooser_add_thumbnail_preview(GTK_FILE_CHOOSER(chooser));
+#else
     preview = gtk_image_new();
     gtk_widget_show(preview);
     gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(chooser), preview);
     gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(chooser), FALSE);
     g_signal_connect(G_OBJECT(chooser), "update-preview",
                      G_CALLBACK(update_preview_cb), preview);
+#endif
     
     gtk_widget_show(chooser);
     if(gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_ACCEPT) {
@@ -783,6 +834,8 @@ static void
 manage_desktop_chk_toggled_cb(GtkToggleButton *tb, gpointer user_data)
 {
     BackdropDialog *bd = user_data;
+    const gchar *setting_name;
+    McsSetting *setting = NULL;
     
     if(gtk_toggle_button_get_active(tb)) {
         GError *err = NULL;
@@ -810,51 +863,59 @@ manage_desktop_chk_toggled_cb(GtkToggleButton *tb, gpointer user_data)
             g_free(secondary);
         } else
             gtk_widget_set_sensitive(bd->top_notebook, TRUE);
+        
+        setting_name = "managedesktop-show-warning-on";
     } else {
         Window xid;
-        McsSetting *setting;
         
         if(xfdesktop_check_is_running(&xid))
             xfdesktop_send_client_message(xid, QUIT_MESSAGE);
         gtk_widget_set_sensitive(bd->top_notebook, FALSE);
         
-        setting = mcs_manager_setting_lookup(bd->plugin->manager,
-                "managedesktop-show-warning", BACKDROP_CHANNEL);
-        if(!setting || setting->data.v_int) {
-            GtkWidget *dlg, *lbl, *chk, *vbox;
-            
-            dlg = gtk_dialog_new_with_buttons(_("Information"),
-                    GTK_WINDOW(bd->dialog),
-                    GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
-                    GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-            
-            vbox = gtk_vbox_new(FALSE, BORDER);
-            gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER);
-            gtk_widget_show(vbox);
-            gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), vbox, TRUE, TRUE, 0);
-            
+        setting_name = "managedesktop-show-warning";
+    }
+    
+    setting = mcs_manager_setting_lookup(bd->plugin->manager,
+            setting_name, BACKDROP_CHANNEL);
+    if(!setting || setting->data.v_int) {
+        GtkWidget *dlg, *lbl, *chk, *vbox;
+        
+        dlg = gtk_dialog_new_with_buttons(_("Information"),
+                GTK_WINDOW(bd->dialog),
+                GTK_DIALOG_DESTROY_WITH_PARENT|GTK_DIALOG_NO_SEPARATOR,
+                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+        
+        vbox = gtk_vbox_new(FALSE, BORDER);
+        gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER);
+        gtk_widget_show(vbox);
+        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), vbox, TRUE, TRUE, 0);
+        
+        /* FIXME: i'm avoiding changing a string here.  after the string is
+         * translated, remove the second string and replace it with the first */
+        if(!strcmp(setting_name, "managedesktp-show-warning-on"))
+            lbl = gtk_label_new(_("To ensure that this setting takes effect the next time you start Xfce, please be sure to save your session when logging out.  If you are not using the Xfce Session Manager (xfce4-session), you will need to manually edit your ~/.config/xfce4/xinitrc file.  Details are available in the documentation provided on http://xfce.org/."));
+        else
             lbl = gtk_label_new(_("To ensure that Xfce does not manage your desktop the next time you start Xfce, please be sure to save your session when logging out.  If you are not using the Xfce Session Manager (xfce4-session), you will need to manually edit your ~/.config/xfce4/xinitrc file.  Details are available in the documentation provided on http://xfce.org/."));
-            gtk_label_set_line_wrap(GTK_LABEL(lbl), TRUE);
-            gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
-            gtk_widget_show(lbl);
-            gtk_box_pack_start(GTK_BOX(vbox), lbl, FALSE, FALSE, 0);
-            
-            add_spacer(GTK_BOX(vbox));
-            
-            chk = gtk_check_button_new_with_mnemonic(_("_Do not show this again"));
-            gtk_widget_show(chk);
-            gtk_box_pack_start(GTK_BOX(vbox), chk, FALSE, FALSE, 0);
-            
-            gtk_dialog_run(GTK_DIALOG(dlg));
-            
-            if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chk))) {
-                mcs_manager_set_int(bd->plugin->manager,
-                        "managedesktop-show-warning", BACKDROP_CHANNEL, 0);
-                backdrop_write_options(bd->plugin);
-            }
-            
-            gtk_widget_destroy(dlg);
+        gtk_label_set_line_wrap(GTK_LABEL(lbl), TRUE);
+        gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+        gtk_widget_show(lbl);
+        gtk_box_pack_start(GTK_BOX(vbox), lbl, FALSE, FALSE, 0);
+        
+        add_spacer(GTK_BOX(vbox));
+        
+        chk = gtk_check_button_new_with_mnemonic(_("_Do not show this again"));
+        gtk_widget_show(chk);
+        gtk_box_pack_start(GTK_BOX(vbox), chk, FALSE, FALSE, 0);
+        
+        gtk_dialog_run(GTK_DIALOG(dlg));
+        
+        if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chk))) {
+            mcs_manager_set_int(bd->plugin->manager, setting_name,
+                                BACKDROP_CHANNEL, 0);
+            backdrop_write_options(bd->plugin);
         }
+        
+        gtk_widget_destroy(dlg);
     }
 }
 
