@@ -41,15 +41,13 @@
 
 #include <libxfcegui4/libxfcegui4.h>
 
+#include <dbus/dbus-glib.h>
+
 #ifdef HAVE_THUNARX
 #include <thunarx/thunarx.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
 #endif
 
 #include "xfdesktop-file-utils.h"
-#include "xfdesktop-icon.h"
-#include "xfdesktop-file-icon.h"
 #include "xfdesktop-dbus-bindings-trash.h"
 #include "xfdesktop-special-file-icon.h"
 
@@ -64,20 +62,21 @@ struct _XfdesktopSpecialFileIconPrivate
     
     /* only needed for trash */
     DBusGProxy *dbus_proxy;
+    DBusGProxyCall *dbus_querytrash_call;
     gboolean trash_full;
 };
 
 static void xfdesktop_special_file_icon_finalize(GObject *obj);
 
-static void xfdesktop_special_file_icon_icon_init(XfdesktopIconIface *iface);
 static GdkPixbuf *xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
                                                           gint size);
 static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
 static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
-static gboolean xfdesktop_special_file_icon_is_drop_dest(XfdesktopIcon *icon);
-static XfdesktopIconDragResult xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
-                                                                        XfdesktopIcon *src_icon,
-                                                                        GdkDragAction action);
+static GdkDragAction xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
+                                                         XfdesktopIcon *src_icon,
+                                                         GdkDragAction action);
 static GtkWidget *xfdesktop_special_file_icon_get_popup_menu(XfdesktopIcon *icon);
 
 static G_CONST_RETURN ThunarVfsInfo *xfdesktop_special_file_icon_peek_info(XfdesktopFileIcon *icon);
@@ -88,21 +87,20 @@ static void xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface);
 
 
 static inline void xfdesktop_special_file_icon_invalidate_pixbuf(XfdesktopSpecialFileIcon *icon);
+static void xfdesktop_special_file_icon_trash_changed_cb(DBusGProxy *proxy,
+                                                         gboolean trash_full,
+                                                         gpointer user_data);
 
 
 #ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
-                       G_IMPLEMENT_INTERFACE(XFDESKTOP_TYPE_ICON,
-                                             xfdesktop_special_file_icon_icon_init)
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
                                              xfdesktop_special_file_icon_tfi_init)
                        )
 #else
-G_DEFINE_TYPE_EXTENDED(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
-                       XFDESKTOP_TYPE_FILE_ICON, 0,
-                       G_IMPLEMENT_INTERFACE(XFDESKTOP_TYPE_ICON,
-                                             xfdesktop_special_file_icon_icon_init)
+G_DEFINE_TYPE(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
+              XFDESKTOP_TYPE_FILE_ICON)
 #endif
 
 
@@ -111,12 +109,21 @@ static void
 xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
+    XfdesktopIconClass *icon_class = (XfdesktopIconClass *)klass;
     XfdesktopFileIconClass *file_icon_class = (XfdesktopFileIconClass *)klass;
     
     g_type_class_add_private(klass, sizeof(XfdesktopSpecialFileIconPrivate));
     
     gobject_class->finalize = xfdesktop_special_file_icon_finalize;
     
+    icon_class->peek_pixbuf = xfdesktop_special_file_icon_peek_pixbuf;
+    icon_class->peek_label = xfdesktop_special_file_icon_peek_label;
+    icon_class->peek_tooltip = xfdesktop_special_file_icon_peek_tooltip;
+    icon_class->get_allowed_drag_actions = xfdesktop_special_file_icon_get_allowed_drag_actions;
+    icon_class->get_allowed_drop_actions = xfdesktop_special_file_icon_get_allowed_drop_actions;
+    icon_class->do_drop_dest = xfdesktop_special_file_icon_do_drop_dest;
+    icon_class->get_popup_menu = xfdesktop_special_file_icon_get_popup_menu;
+    
     file_icon_class->peek_info = xfdesktop_special_file_icon_peek_info;
     file_icon_class->can_rename_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
     file_icon_class->can_delete_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
@@ -140,6 +147,17 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
                                          G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
                                          icon);
     
+    if(icon->priv->dbus_proxy) {
+        if(icon->priv->dbus_querytrash_call) {
+            dbus_g_proxy_cancel_call(icon->priv->dbus_proxy,
+                                     icon->priv->dbus_querytrash_call);
+        }
+        dbus_g_proxy_disconnect_signal(icon->priv->dbus_proxy, "TrashChanged",
+                                       G_CALLBACK(xfdesktop_special_file_icon_trash_changed_cb),
+                                       icon);
+        g_object_unref(G_OBJECT(icon->priv->dbus_proxy));
+    }
+    
     if(icon->priv->pix)
         g_object_unref(G_OBJECT(icon->priv->pix));
     
@@ -152,18 +170,6 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
     G_OBJECT_CLASS(xfdesktop_special_file_icon_parent_class)->finalize(obj);
 }
 
-static void
-xfdesktop_special_file_icon_icon_init(XfdesktopIconIface *iface)
-{
-    iface->peek_pixbuf = xfdesktop_special_file_icon_peek_pixbuf;
-    iface->peek_label = xfdesktop_special_file_icon_peek_label;
-    iface->peek_tooltip = xfdesktop_special_file_icon_peek_tooltip;
-    iface->is_drop_dest = xfdesktop_special_file_icon_is_drop_dest;
-    iface->do_drop_dest = xfdesktop_special_file_icon_do_drop_dest;
-    iface->get_popup_menu = xfdesktop_special_file_icon_get_popup_menu;
-}
-
-
 #ifdef HAVE_THUNARX
 static gchar *
 xfdesktop_special_file_icon_tfi_get_uri_scheme(ThunarxFileInfo *file_info)
@@ -217,6 +223,18 @@ xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
                 custom_icon_name = "user-home";
             else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-desktop"))
                 custom_icon_name = "gnome-fs-desktop";
+        } else if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == file_icon->priv->type) {
+            if(file_icon->priv->trash_full) {
+                if(gtk_icon_theme_has_icon(icon_theme, "user-trash-full"))
+                    custom_icon_name = "user-trash-full";
+                else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-trash-full"))
+                    custom_icon_name = "gnome-fs-trash-full";
+            } else {
+                if(gtk_icon_theme_has_icon(icon_theme, "user-trash"))
+                    custom_icon_name = "user-trash";
+                else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-trash-empty"))
+                    custom_icon_name = "gnome-fs-trash-empty";
+            }
         }
         
         file_icon->priv->pix = xfdesktop_file_utils_get_file_icon(custom_icon_name,
@@ -242,12 +260,64 @@ xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon)
         return special_file_icon->priv->info->display_name;
 }
 
-static gboolean
-xfdesktop_special_file_icon_is_drop_dest(XfdesktopIcon *icon)
+static GdkDragAction
+xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    /* FIXME: i suppose '/' could be writable if we're foolishly running as root */
-    return (XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM != special_file_icon->priv->type);
+    GdkDragAction actions = 0;
+    
+    switch(special_file_icon->priv->type) {
+        case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
+            /* move is just impossible, and copy seems a bit retarded.  link
+             * is possible, but thunar-vfs doesn't support it (deliberately). */
+            actions = 0;
+            break;
+        
+        case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
+            /* user shouldn't be able to move their own homedir.  copy might
+             * be a little silly, but allow it anyway.  link is fine. */
+            actions = GDK_ACTION_COPY | GDK_ACTION_LINK;
+            break;
+            
+        case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
+            /* i don't think we can even do a link here; thunar doesn't let
+             * us, anyway. */
+            actions = 0;
+            break;
+    }
+    
+    return actions;
+}
+
+static GdkDragAction
+xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+{
+    XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
+    const ThunarVfsInfo *info;
+    GdkDragAction actions = 0;
+    
+    switch(special_file_icon->priv->type) {
+        case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
+            /* we should hope the user isn't running as root, but we might as
+             * well let them hang themselves if they are */
+            info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
+            if(info && info->flags & THUNAR_VFS_FILE_FLAGS_WRITABLE)
+                actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+            else
+                actions = 0;
+            break;
+        
+        case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
+            /* assume the user can write to their own home directory */
+            actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+            break;
+            
+        case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
+            actions = GDK_ACTION_MOVE;  /* anything else is just silly */
+            break;
+    }
+    
+    return actions;
 }
 
 static void
@@ -275,7 +345,10 @@ xfdesktop_special_file_icon_interactive_job_ask(ThunarVfsJob *job,
                                                 ThunarVfsInteractiveJobResponse choices,
                                                 gpointer user_data)
 {
-    return xfdesktop_file_utils_interactive_job_ask(NULL, message, choices);
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(user_data));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
+    return xfdesktop_file_utils_interactive_job_ask(GTK_WINDOW(toplevel),
+                                                    message, choices);
 }
 
 static void
@@ -298,7 +371,7 @@ xfdesktop_special_file_icon_drag_job_finished(ThunarVfsJob *job,
     g_object_unref(G_OBJECT(special_file_icon));
 }
 
-static XfdesktopIconDragResult
+static gboolean
 xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                          XfdesktopIcon *src_icon,
                                          GdkDragAction action)
@@ -307,26 +380,29 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     XfdesktopFileIcon *src_file_icon = XFDESKTOP_FILE_ICON(src_icon);
     const ThunarVfsInfo *src_info;
     ThunarVfsJob *job = NULL;
-    const gchar *name;
+    const gchar *name = NULL;
     ThunarVfsPath *dest_path;
     
     DBG("entering");
     
     g_return_val_if_fail(special_file_icon && src_file_icon,
-                         XFDESKTOP_ICON_DRAG_FAILED);
-    g_return_val_if_fail(xfdesktop_special_file_icon_is_drop_dest(icon),
-                         XFDESKTOP_ICON_DRAG_FAILED);
+                         FALSE);
+    g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon),
+                         FALSE);
     
     src_info = xfdesktop_file_icon_peek_info(src_file_icon);
     if(!src_info)
-        return XFDESKTOP_ICON_DRAG_FAILED;
+        return FALSE;
+    
+    if(thunar_vfs_path_is_root(src_info->path))
+        return FALSE;
     
     name = thunar_vfs_path_get_name(src_info->path);
-    g_return_val_if_fail(name, XFDESKTOP_ICON_DRAG_FAILED);
-        
+    g_return_val_if_fail(name, FALSE);
+    
     dest_path = thunar_vfs_path_relative(special_file_icon->priv->info->path,
                                          name);
-    g_return_val_if_fail(dest_path, XFDESKTOP_ICON_DRAG_FAILED);
+    g_return_val_if_fail(dest_path, FALSE);
     
     if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH)  {
         /* any drop to the trash is a move */
@@ -387,11 +463,11 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                          G_CALLBACK(xfdesktop_special_file_icon_drag_job_finished),
                          special_file_icon);
         
-        return XFDESKTOP_ICON_DRAG_SUCCEEDED_NO_ACTION;
+        return TRUE;
     } else
-        return XFDESKTOP_ICON_DRAG_FAILED;
+        return FALSE;
     
-    return XFDESKTOP_ICON_DRAG_FAILED;
+    return FALSE;
 }
 
 static G_CONST_RETURN gchar *
@@ -422,17 +498,19 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 }
 
 static void
-xfdesktop_special_file_icon_trash_handle_error(GdkScreen *gscreen,
+xfdesktop_special_file_icon_trash_handle_error(XfdesktopSpecialFileIcon *icon,
                                                const gchar *method,
                                                const gchar *message)
 {
-    GtkWidget *dlg = xfce_message_dialog_new(NULL, _("Trash Error"),
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
+    GtkWidget *dlg = xfce_message_dialog_new(GTK_WINDOW(toplevel),
+                                             _("Trash Error"),
                                              GTK_STOCK_DIALOG_WARNING,
                                              _("Unable to contact the Xfce Trash service."),
                                              _("Make sure you have a file manager installed that supports the Xfce Trash service, such as Thunar."),
                                              GTK_STOCK_CLOSE,
                                              GTK_RESPONSE_ACCEPT, NULL);
-    gtk_window_set_screen(GTK_WINDOW(dlg), gscreen);
     gtk_dialog_run(GTK_DIALOG(dlg));
     gtk_widget_destroy(dlg);
     
@@ -445,11 +523,20 @@ xfdesktop_special_file_icon_trash_open_cb(DBusGProxy *proxy,
                                           GError *error,
                                           gpointer user_data)
 {
+    GtkWidget *icon_view, *toplevel;
+    
+    icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(user_data));
+    toplevel = gtk_widget_get_toplevel(icon_view);
+    xfdesktop_file_utils_set_window_cursor(GTK_WINDOW(toplevel),
+                                           GDK_LEFT_PTR);
+    
     if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(GDK_SCREEN(user_data),
+        xfdesktop_special_file_icon_trash_handle_error(XFDESKTOP_SPECIAL_FILE_ICON(user_data),
                                                        "DisplayTrash",
                                                         error->message);
     }
+    
+    g_object_unref(G_OBJECT(user_data));
 }
 
 static void
@@ -458,10 +545,12 @@ xfdesktop_special_file_icon_trash_empty_cb(DBusGProxy *proxy,
                                            gpointer user_data)
 {
     if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(GDK_SCREEN(user_data),
+        xfdesktop_special_file_icon_trash_handle_error(XFDESKTOP_SPECIAL_FILE_ICON(user_data),
                                                        "EmptyTrash",
                                                         error->message);
     }
+    
+    g_object_unref(G_OBJECT(user_data));
 }
 
 static void
@@ -476,12 +565,22 @@ xfdesktop_special_file_icon_trash_open(GtkWidget *w,
         if(!org_xfce_Trash_display_trash_async(file_icon->priv->dbus_proxy,
                                                display_name,
                                                xfdesktop_special_file_icon_trash_open_cb,
-                                               file_icon->priv->gscreen))
+                                               file_icon))
         {
-            xfdesktop_special_file_icon_trash_handle_error(file_icon->priv->gscreen,
+            xfdesktop_special_file_icon_trash_handle_error(file_icon,
                                                            "DisplayTrash",
                                                             NULL);
+        } else {
+            GtkWidget *icon_view, *toplevel;
+            
+            g_object_ref(G_OBJECT(file_icon));
+            
+            icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(file_icon));
+            toplevel = gtk_widget_get_toplevel(icon_view);
+            xfdesktop_file_utils_set_window_cursor(GTK_WINDOW(toplevel),
+                                                   GDK_WATCH);
         }
+            
         g_free(display_name);
     }
 }
@@ -498,12 +597,13 @@ xfdesktop_special_file_icon_trash_empty(GtkWidget *w,
         if(!org_xfce_Trash_empty_trash_async(file_icon->priv->dbus_proxy,
                                              display_name,
                                              xfdesktop_special_file_icon_trash_empty_cb,
-                                             file_icon->priv->gscreen))
+                                             file_icon))
         {
-            xfdesktop_special_file_icon_trash_handle_error(file_icon->priv->gscreen,
+            xfdesktop_special_file_icon_trash_handle_error(file_icon,
                                                            "EmptyTrash",
                                                             NULL);
-        }
+        } else
+            g_object_ref(G_OBJECT(file_icon));
         g_free(display_name);
     }
 }
@@ -581,11 +681,16 @@ xfdesktop_special_file_icon_query_trash_cb(DBusGProxy *proxy,
     XfdesktopSpecialFileIcon *icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
     
     if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(icon->priv->gscreen,
+        xfdesktop_special_file_icon_trash_handle_error(icon,
                                                        "QueryTrash",
                                                        error->message);
-    } else
+    } else {
         icon->priv->trash_full = trash_full;
+        xfdesktop_special_file_icon_invalidate_pixbuf(icon);
+        xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
+    }
+    
+    icon->priv->dbus_querytrash_call = NULL;
 }
 
 
@@ -626,39 +731,26 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
     }
     
     if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == type) {
-        DBusGConnection *dgconn = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-        
-        if(G_LIKELY(dgconn)) {
-            /* dbus's default is brain-dead */
-            DBusConnection *dconn = dbus_g_connection_get_connection(dgconn);
-            dbus_connection_set_exit_on_disconnect(dconn, FALSE);
+        DBusGProxy *trash_proxy = xfdesktop_file_utils_peek_trash_proxy();
             
-            special_file_icon->priv->dbus_proxy = dbus_g_proxy_new_for_name(dgconn,
-                                                                            "org.xfce.FileManager",
-                                                                            "/org/xfce/FileManager",
-                                                                            "org.xfce.Trash");
+        if(G_LIKELY(trash_proxy)) {
+            DBusGProxyCall *call;
             
-            dbus_g_proxy_add_signal(special_file_icon->priv->dbus_proxy,
-                                    "TrashChanged", G_TYPE_BOOLEAN,
-                                    G_TYPE_INVALID);
+            special_file_icon->priv->dbus_proxy = g_object_ref(G_OBJECT(trash_proxy));
             dbus_g_proxy_connect_signal(special_file_icon->priv->dbus_proxy,
                                         "TrashChanged",
                                         G_CALLBACK(xfdesktop_special_file_icon_trash_changed_cb),
                                         special_file_icon, NULL);
             
-            if(!org_xfce_Trash_query_trash_async(special_file_icon->priv->dbus_proxy,
-                                                 xfdesktop_special_file_icon_query_trash_cb,
-                                                 special_file_icon))
-            {
-                xfdesktop_special_file_icon_trash_handle_error(special_file_icon->priv->gscreen,
+            call = org_xfce_Trash_query_trash_async(special_file_icon->priv->dbus_proxy,
+                                                    xfdesktop_special_file_icon_query_trash_cb,
+                                                    special_file_icon);
+            if(!call) {
+                xfdesktop_special_file_icon_trash_handle_error(special_file_icon,
                                                                "QueryTrash",
                                                                NULL);
             }
-            
-            /* the DBusGProxy (actually, its associated DBusGProxyManager) holds
-             * a reference to the DBusGConnection during its lifetime, so we
-             * don't have to hold a ref. */
-            dbus_g_connection_unref(dgconn);
+            special_file_icon->priv->dbus_querytrash_call = call;
         } else {
             /* we might as well just bail here */
             g_object_unref(G_OBJECT(special_file_icon));
